package com.example.cameraxtestappjava;

import static com.example.cameraxtestappjava.segpass.SegpassCamera.REQUEST_CAMERA_PERMISSION;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;

import androidx.activity.EdgeToEdge;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import com.example.cameraxtestappjava.databinding.ActivitySegpassCameraBinding;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassCameraCallback;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassPermissionCallback;
import com.example.cameraxtestappjava.segpass.camera.view.SegpassCameraLayout;

public class SegpassCameraActivity extends AppCompatActivity implements ActivityCompat.OnRequestPermissionsResultCallback, SegpassCameraCallback, SegpassPermissionCallback  {

    ActivitySegpassCameraBinding binding;
    SegpassCameraLayout cameraLayout;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);

        binding = ActivitySegpassCameraBinding.inflate(getLayoutInflater());
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(binding.getRoot());

        setCamera();

        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });
    }

    private void setCamera() {
        cameraLayout = binding.sclCameraLayout; // Bind SegpassCameraLayout to element in view
        cameraLayout.init(this, this, this); // Init SegpassCamera
    }

    @Override
    public void onResume() {
        super.onResume();
        cameraLayout.resumeCamera();
    }

    @Override
    public void onPause() {
        cameraLayout.pauseCamera();
        super.onPause();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        if (requestCode == REQUEST_CAMERA_PERMISSION) {
            if (grantResults.length != 2 || grantResults[0] != PackageManager.PERMISSION_GRANTED) {
                onPermissionDenied();
            }
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    /**
     * Camera callback
     */

    @Override
    public void onPictureTakenSuccess(String base64) {
        cameraLayout.showToast("Base64 generated.");
        this.finish();
    }

    @Override
    public void onPictureTakenFail(String error) {
        cameraLayout.showToast(error);
        this.finish();
    }

    /**
     * Camera state callback (just to validate if there was an error while initializing the camera)
     */

    @Override
    public void onCameraInitError(String errorMessage) {
        cameraLayout.showToast(errorMessage);
        this.finish();
    }

    /**
     * Permissions callbck
     */

    @Override
    public void onPermissionRequest() {
        requestPermissions(new String[]{android.Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_CAMERA_PERMISSION);
    }

    @Override
    public void onPermissionGranted() {
        // Do nothing, use camera.
    }

    @Override
    public void onPermissionDenied() {
        cameraLayout.showToast("No permissions granted, closing camera.");
        finish();
    }
}