package com.example.cameraxtestappjava.segpass.camera.utils;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.Image;
import android.util.Base64;
import android.util.Log;

import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;

public class ImageSaverUtil implements Runnable {

    /**
     * Tag for the {@link Log}
     */
    private static final String TAG = "ImageSaverUtil";

    /**
     * The JPEG image
     */
    private final Image mImage;

    /**
     * Callback to return encoded image value in {@link Base64}
     */
    private final ImageEncodingCallback mCallback;

    public ImageSaverUtil(Image image, ImageEncodingCallback callback) {
        mImage = image;
        mCallback = callback;
    }

    @Override
    public void run() {
        try {

            // Get image as byte data buffer
            ByteBuffer buffer = mImage.getPlanes()[0].getBuffer();
            byte[] bytes = new byte[buffer.capacity()];
            buffer.get(bytes);

            // Convert byte data to a Bitmap
            Bitmap bitmap = BitmapFactory.decodeByteArray(bytes, 0, bytes.length);

            // Create a ByteArrayOutputStream to capture the compressed image data
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

            // Compress the bitmap with JPEG format and desired quality (25% will do for this use case)
            bitmap.compress(Bitmap.CompressFormat.JPEG, 25, outputStream);

            // Get the compressed image data as a byte array
            byte[] compressedImageData = outputStream.toByteArray();

            // Encode the compressed data to Base64 string
            String base64String = Base64.encodeToString(compressedImageData, Base64.NO_WRAP);

            // Close the stream (optional, as ByteArrayOutputStream should close automatically)
            outputStream.close();

            // Return Base64 string on callback
            mCallback.onImageEncodeSuccess(base64String);

        } catch (Exception e) {
            Log.e(TAG, "Error encoding photo: " + e.getMessage());
            mCallback.onImageEncodeFail(e.getMessage());
        }
    }
}
