package com.example.cameraxtestappjava.segpass.camera.view;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.example.cameraxtestappjava.R;
import com.example.cameraxtestappjava.segpass.SegpassCamera;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassCameraCallback;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassPermissionCallback;

public class SegpassCameraLayout extends FrameLayout {

    /**
     * Parent {@link AppCompatActivity} where the SegpassCameraLayout is instantiated from.
     */
    private AppCompatActivity mActivity;

    /**
     * Callback for camera permissions
     */
    private SegpassPermissionCallback mPermissionListener;

    /**
     * Callback dealing with camera state and results.
     */
    private SegpassCameraCallback mCameraCallback;

    /**
     * SegpassCamera
     */
    private SegpassCamera mCamera;

    /**
     * SegpassCameraLayout root view.
     */
    private final View rootView;

    /**
     *
     */

    private ImageButton mTakePicture;
    private ImageButton mDiscardPicture;
    private ImageButton mSavePicture;

    public SegpassCameraLayout(@NonNull Context context) {
        super(context);
        rootView = inflate(context, R.layout.segpass_camera_layout, this);
    }

    public SegpassCameraLayout(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        rootView = inflate(context, R.layout.segpass_camera_layout, this);
    }

    /**
     * @param activity       Parent activity where the camera is called from.
     * @param listener       {@link SegpassPermissionCallback} that deals with camera and storage permissions.
     * @param cameraCallback {@link SegpassCameraCallback} that deals with operation results.
     */
    public void init(AppCompatActivity activity, SegpassPermissionCallback listener, SegpassCameraCallback cameraCallback) {
        mActivity = activity;
        mPermissionListener = listener;
        mCameraCallback = cameraCallback;
        setUpCamera();
        setUpViews();
        setUpListeners();
    }

    /**
     * Set up {@link AutoFitTextureView} and {@link SegpassCamera}
     */
    private void setUpCamera() {
        AutoFitTextureView mTextureView = rootView.findViewById(R.id.tvCameraTextureView);
        mCamera = new SegpassCamera(mActivity, mTextureView, mPermissionListener, mCameraCallback);
        mCamera.resumeCamera();
    }

    private void setUpViews() {
        mTakePicture = rootView.findViewById(R.id.btnTakePhoto);
        mDiscardPicture = rootView.findViewById(R.id.btnDiscardPicture);
        mSavePicture = rootView.findViewById(R.id.btnSavePicture);
    }

    private void setUpListeners() {
        mTakePicture.setOnClickListener(v -> {
            showPictureButtons();
            hideShutterButton();
            mCamera.takePicture();
        });
        mDiscardPicture.setOnClickListener(v -> {
            showShutterButton();
            hidePictureButtons();
            mCamera.discardPicture();
        });
        mSavePicture.setOnClickListener(v -> {
            showShutterButton();
            hidePictureButtons();
            mCamera.savePicture();
        });
    }

    public void resumeCamera() {
        mCamera.resumeCamera();
    }

    public void pauseCamera() {
        mCamera.pauseCamera();
    }

    public void showToast(String message) {
        mCamera.showToast(message);
    }

    private void showShutterButton() {
        mTakePicture.setVisibility(View.VISIBLE);
    }

    private void hideShutterButton() {
        mTakePicture.setVisibility(View.GONE);
    }

    private void showPictureButtons() {
        mSavePicture.setVisibility(View.VISIBLE);
        mDiscardPicture.setVisibility(View.VISIBLE);
    }

    private void hidePictureButtons() {
        mSavePicture.setVisibility(View.GONE);
        mDiscardPicture.setVisibility(View.GONE);
    }
}
